CREATE DATABASE cluebringer;
USE cluebringer;
SET FOREIGN_KEY_CHECKS=0;
CREATE TABLE policies (
	ID			SERIAL,
	Name			VARCHAR(255) NOT NULL,
	Priority		SMALLINT NOT NULL,
	Description		TEXT,
	Disabled		SMALLINT NOT NULL DEFAULT '0'
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE policy_members (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Source			TEXT,
	Destination		TEXT,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE policy_groups (
	ID			SERIAL,
	Name			VARCHAR(255) NOT NULL,
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	Comment			VARCHAR(1024),
	UNIQUE (Name)
)  ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE policy_group_members (
	ID			SERIAL,
	PolicyGroupID		BIGINT UNSIGNED,
	Member			VARCHAR(255) NOT NULL,
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	Comment			VARCHAR(1024),
	FOREIGN KEY (PolicyGroupID) REFERENCES policy_groups(ID)
)  ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE session_tracking (
	Instance		VARCHAR(255),
	QueueID			VARCHAR(255),
	Timestamp		BIGINT NOT NULL,
	ClientAddress		VARCHAR(64),
	ClientName		VARCHAR(255),
	ClientReverseName	VARCHAR(255),
	Protocol		VARCHAR(255),
	EncryptionProtocol	VARCHAR(255),
	EncryptionCipher	VARCHAR(255),
	EncryptionKeySize	VARCHAR(255),
	SASLMethod		VARCHAR(255),
	SASLSender		VARCHAR(255),
	SASLUsername		VARCHAR(255),
	Helo			VARCHAR(255),
	Sender			VARCHAR(255),
	Size			BIGINT UNSIGNED,
	RecipientData		TEXT,  /* Policy state information */
	UNIQUE (Instance)
)  ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE access_control (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	Verdict			VARCHAR(255),
	Data			TEXT,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE checkhelo (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	UseBlacklist			SMALLINT,
	BlacklistPeriod			BIGINT UNSIGNED,
	UseHRP				SMALLINT,
	HRPPeriod			BIGINT UNSIGNED,
	HRPLimit			BIGINT UNSIGNED,
	RejectInvalid			SMALLINT,
	RejectIP			SMALLINT,
	RejectUnresolvable		SMALLINT,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE checkhelo_blacklist (
	ID			SERIAL,
	Helo			VARCHAR(255) NOT NULL,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	UNIQUE (Helo)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE checkhelo_whitelist (
	ID			SERIAL,
	Source			VARCHAR(512) NOT NULL,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	UNIQUE (Source)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE checkhelo_tracking (
	Address			VARCHAR(64) NOT NULL,
	Helo			VARCHAR(255) NOT NULL,
	LastUpdate		BIGINT UNSIGNED NOT NULL,
	UNIQUE (Address,Helo)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE checkspf (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	UseSPF				SMALLINT,
	RejectFailedSPF			SMALLINT,
	AddSPFHeader			SMALLINT,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE quotas (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	Track			VARCHAR(255) NOT NULL,
	Period			BIGINT UNSIGNED,
	Verdict			VARCHAR(255),
	Data			TEXT,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE quotas_limits (
	ID			SERIAL,
	QuotasID		BIGINT UNSIGNED,
	Type			VARCHAR(255),
	CounterLimit		BIGINT UNSIGNED,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (QuotasID) REFERENCES quotas(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE quotas_tracking (
	QuotasLimitsID		BIGINT UNSIGNED,
	TrackKey		VARCHAR(512),
	LastUpdate		BIGINT UNSIGNED,
  Counter			NUMERIC(10,4),
	UNIQUE (QuotasLimitsID,TrackKey),
	FOREIGN KEY (QuotasLimitsID) REFERENCES quotas_limits(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE amavis_rules (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	bypass_virus_checks	SMALLINT,
	bypass_virus_checks_m	SMALLINT NOT NULL DEFAULT '0',
	bypass_banned_checks	SMALLINT,
	bypass_banned_checks_m	SMALLINT NOT NULL DEFAULT '0',
	bypass_spam_checks	SMALLINT,
	bypass_spam_checks_m	SMALLINT NOT NULL DEFAULT '0',
	bypass_header_checks	SMALLINT,
	bypass_header_checks_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag_level		FLOAT,
	spam_tag_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag2_level		FLOAT,
	spam_tag2_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag3_level		FLOAT,
	spam_tag3_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_kill_level		FLOAT,
	spam_kill_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_dsn_cutoff_level	FLOAT,
	spam_dsn_cutoff_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_quarantine_cutoff_level	FLOAT,
	spam_quarantine_cutoff_level_m	SMALLINT NOT NULL DEFAULT '0',
	spam_modifies_subject	SMALLINT,
	spam_modifies_subject_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag_subject	VARCHAR(255),
	spam_tag_subject_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag2_subject	VARCHAR(255),
	spam_tag2_subject_m	SMALLINT NOT NULL DEFAULT '0',
	spam_tag3_subject	VARCHAR(255),
	spam_tag3_subject_m	SMALLINT NOT NULL DEFAULT '0',
	max_message_size	BIGINT,
	max_message_size_m	SMALLINT NOT NULL DEFAULT '0',
	banned_files		TEXT,
	banned_files_m		SMALLINT NOT NULL DEFAULT '0',
	sender_whitelist	TEXT,
	sender_whitelist_m	SMALLINT NOT NULL DEFAULT '0',
	sender_blacklist	TEXT,
	sender_blacklist_m	SMALLINT NOT NULL DEFAULT '0',
	notify_admin_newvirus	VARCHAR(255),
	notify_admin_newvirus_m	SMALLINT NOT NULL DEFAULT '0',
	notify_admin_virus	VARCHAR(255),
	notify_admin_virus_m	SMALLINT NOT NULL DEFAULT '0',
	notify_admin_spam	VARCHAR(255),
	notify_admin_spam_m	SMALLINT NOT NULL DEFAULT '0',
	notify_admin_banned_file	VARCHAR(255),
	notify_admin_banned_file_m	SMALLINT NOT NULL DEFAULT '0',
	notify_admin_bad_header	VARCHAR(255),
	notify_admin_bad_header_m	SMALLINT NOT NULL DEFAULT '0',
	quarantine_virus	VARCHAR(255),
	quarantine_virus_m	SMALLINT NOT NULL DEFAULT '0',
	quarantine_banned_file	VARCHAR(255),
	quarantine_banned_file_m	SMALLINT NOT NULL DEFAULT '0',
	quarantine_bad_header	VARCHAR(255),
	quarantine_bad_header_m	SMALLINT NOT NULL DEFAULT '0',
	quarantine_spam		VARCHAR(255),
	quarantine_spam_m	SMALLINT NOT NULL DEFAULT '0',
	bcc_to			VARCHAR(255),
	bcc_to_m		SMALLINT NOT NULL DEFAULT '0',
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE greylisting (
	ID			SERIAL,
	PolicyID		BIGINT UNSIGNED,
	Name			VARCHAR(255) NOT NULL,
	UseGreylisting			SMALLINT,
	GreylistPeriod			BIGINT UNSIGNED,
	Track				VARCHAR(255) NOT NULL,
	GreylistAuthValidity		BIGINT UNSIGNED,
	GreylistUnAuthValidity		BIGINT UNSIGNED,
	UseAutoWhitelist		SMALLINT,
	AutoWhitelistPeriod		BIGINT UNSIGNED,
	AutoWhitelistCount		BIGINT UNSIGNED,
	AutoWhitelistPercentage		BIGINT UNSIGNED,
	UseAutoBlacklist		SMALLINT,
	AutoBlacklistPeriod		BIGINT UNSIGNED,
	AutoBlacklistCount		BIGINT UNSIGNED,
	AutoBlacklistPercentage		BIGINT UNSIGNED,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	FOREIGN KEY (PolicyID) REFERENCES policies(ID)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE greylisting_whitelist (
	ID			SERIAL,
	Source			VARCHAR(255) NOT NULL,
	Comment			VARCHAR(1024),
	Disabled		SMALLINT NOT NULL DEFAULT '0',
	UNIQUE (Source)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE greylisting_autowhitelist (
	ID			SERIAL,
	TrackKey		VARCHAR(512) NOT NULL,
	Added			BIGINT UNSIGNED NOT NULL,
	LastSeen		BIGINT UNSIGNED NOT NULL,
	Comment			VARCHAR(1024),
	UNIQUE (TrackKey)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE greylisting_autoblacklist (
	ID			SERIAL,
	TrackKey		VARCHAR(512) NOT NULL,
	Added			BIGINT UNSIGNED NOT NULL,
	Comment			VARCHAR(1024),
	UNIQUE (TrackKey)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE TABLE greylisting_tracking (
	TrackKey		VARCHAR(512) NOT NULL,
	Sender			VARCHAR(255) NOT NULL,
	Recipient		VARCHAR(255) NOT NULL,
	FirstSeen		BIGINT UNSIGNED NOT NULL,
	LastUpdate		BIGINT UNSIGNED NOT NULL,
	Tries			BIGINT UNSIGNED NOT NULL,
	Count			BIGINT UNSIGNED NOT NULL,
	UNIQUE(TrackKey,Sender,Recipient)
) ENGINE=InnoDB CHARACTER SET latin1 COLLATE latin1_bin;
CREATE INDEX session_tracking_idx1 ON session_tracking (QueueID,ClientAddress,Sender);
CREATE INDEX session_tracking_idx2 ON session_tracking (Timestamp);
CREATE INDEX checkhelo_tracking_idx1 ON checkhelo_tracking (LastUpdate);
CREATE INDEX quotas_tracking_idx1 ON quotas_tracking (LastUpdate);
CREATE INDEX greylisting_tracking_idx1 ON greylisting_tracking (LastUpdate,Count);

INSERT INTO policy_groups VALUES(1,'example_quota_group',1,'');
INSERT INTO policy_group_members VALUES(1,1,'$a_username',1,'');

INSERT INTO policy_groups VALUES(2,'special_quotas',0,'');
INSERT INTO policy_group_members VALUES(2,2,'%example_quota_group',1,'');

INSERT INTO policies VALUES(1,'default',5,'500 mail per day',0);
INSERT INTO policy_members VALUES(1,1,'$*,!%special_quotas','any','',0);

INSERT INTO policies VALUES(2,'example policy',10,'example policy',1);
INSERT INTO policy_members VALUES(2,2,'%example_quota_group','any','',1);

INSERT INTO quotas VALUES(1,1,'default quota','SASLUsername',86400,'DEFER','','',0);
INSERT INTO quotas_limits VALUES(1,1,'MessageCount',500,'',0);

INSERT INTO quotas VALUES(2,2,'example quota','SASLUsername',3600,'DEFER','','',1);
INSERT INTO quotas_limits VALUES(2,2,'MessageCount',1000,'',1);

INSERT INTO access_control VALUES(1,1,'trusted senders','OK','','',1);

CREATE USER cluebringer@'%';
GRANT ALL PRIVILEGES ON cluebringer.* TO cluebringer@'%';