#!/usr/bin/ruby.ruby2.5 
#
#
# Bender-NG
#
# Copyright (C) 2007  Windmill Media
# Copyright (c) 2006-2007 Lele Forzani <lele@windmill.it>
#
#
# = bender.rb
#
# <short descr>
#
# Author:: Lele Forzani <lele@windmill.it>
# Documentation::  Lele Forzani <lele@windmill.it>
# License::
#   This is NOT free software. License for using this software is granted
#   by written permission from Lele Forzani <lele@windmill.it>
# Revision:: $Id: $
#
#
# == Description
#
# Main runnable interface
#
# == Annotations
#
#
# == Todo

##
#
#

require 'bender/classes/ICBackend'

require 'getoptlong'

class Bender

  attr_accessor :ac
  attr_accessor :mode

  Version = '1.1'

  def initialize
    @args = {}

    getopts
    @ac = ICBackend.new
  end

  def getopts
    opts = GetoptLong.new(
      [ '--help',      '-h', GetoptLong::NO_ARGUMENT ],
      [ '--provision', '-p', GetoptLong::OPTIONAL_ARGUMENT ],
      [ '--deprovision', '-d', GetoptLong::OPTIONAL_ARGUMENT ],
      [ '--update',    '-u', GetoptLong::OPTIONAL_ARGUMENT ],
      [ '--maint',     '-m', GetoptLong::OPTIONAL_ARGUMENT ],
      [ '--debug',           GetoptLong::NO_ARGUMENT ],
      [ '--info',            GetoptLong::NO_ARGUMENT ],
      [ '--silent',          GetoptLong::NO_ARGUMENT ],
      [ '--service',   '-s', GetoptLong::REQUIRED_ARGUMENT ],
      [ '--force',     '-f', GetoptLong::NO_ARGUMENT ]
    )

    opts.each { |opt, arg |
      case opt
      when '--maint'
        @mode = 'maintain'
        @args = @args.merge({backend: arg.to_s})
      when '--update'
        @mode = 'update'
        @args = @args.merge({backend: arg.to_s})
      when '--provision'
        @mode = 'provision'
        @args = @args.merge({backend: arg.to_s})
      when '--deprovision'
        @mode = 'deprovision'
        @args = @args.merge({backend: arg.to_s})
      when '--silent'
        $silent = true
      when '--debug'
        $debug = true
      when '--info'
        $info = true
      when '--force'
        @args = @args.merge({force: true})
      when '--service'
        @args = @args.merge({service: arg.to_s})
      when '--help'
        help
        exit
      end
    }

    if(!defined?(@mode))
      @mode = 'provision'
    end

  end

  def do
    ac.serviceProvision(mode, @args)
  end


  def version
    ver = ''
    ver += "Bender-NG version #{Bender::Version}\n"
    ver += "(c) 2007 Windmill Media\n"
  end

  def help
    print version
    print("\n---\nbender [mode] [switches]\n")

    print("
    Mode is one of [--provision | --update | --maint]

    --provision [class] service provision, ie, when a new service is added on the host
    --maint [class]     typically run from cron to perform periodical housekeeping
    
      class: docker, webserver, letsencrypt or mysql
    
    Switches are one or more of

    --silent    output is sent to configured log output, usually when running from cron
    --info      print a slighly verbose output for each operation
    --debug     be extremely verbose
    --force     force execution
    --service   run selected service (login or vhost server name)

    --help      here it is

  ")

  end

end



# $URL: $
# $Rev: $
# $Author: $
# $Date: $
# $Id: $

