#
# Bender-NG
#
# Copyright (C) 2007  Windmill Media
# Copyright (c) 2006-2007 Lele Forzani <lele@windmill.it>
#
#
# = ICBackend.rb
#
# <short descr>
#
# Author:: Lele Forzani <lele@windmill.it>
# Documentation::  Lele Forzani <lele@windmill.it>
# License::
#   This is NOT free software. License for using this software is granted
#   by written permission from Lele Forzani <lele@windmill.it>
# Revision:: $Id: ICBackend.rb 1374 2007-07-03 21:19:41Z lele $
#
#
# == Description
#
#
# == Annotations
#
#
# == Todo

require 'bender/tools/Log'
require 'bender/tools/Debug'
require 'bender/tools/ConfigFile'

require 'rubygems'
require 'active_record'

require 'bender/tools/ARValidation'


class ICBackend

  ICAC_CONFIG_ROOT = '/etc/bender'
  #ICAC_IDENT_OVER = 'vhosts'


  attr_accessor :log

  attr_accessor :configRoot
  attr_accessor :baseConfig

  attr_accessor :databaseConfig

  attr_accessor :localID


  def initialize(localID = nil, configRoot=ICAC_CONFIG_ROOT)
    
    @configRoot = configRoot
    getBaseConfig
    getLocalId(localID)

    
    if($log.nil?)
      startLogger
    else
      @log = Log.instance
    end

    ActiveRecord::Base.logger = @log
    
    DEBUG { log.debug("== #{self.class} initializing") }
  end

  #
  # does the real job of provisioning/updating the service
  #
  def serviceProvision(mode='update', args = {})
    backend = args[:backend] || 'webserver'

    log.warn("== Performing operation, mode '#{mode}'")

    log.warn("-- Service #{backend} ---------")
    ## how we implement pluggable services? FIXME
    begin
      require('bender/classes/ServicesBackend/' + backend.underscore.camelize)
      icService = str2class('IC' + backend.underscore.camelize)
    rescue LoadError
      log.error("Service: #{backend} failed initialization of backend: #{$!}")
      exit 1
    end

    svc = icService.new(localID, configRoot)
    if(svc.respond_to?(mode + 'Service'))
      svc.send(mode + 'Service', args)
    end
    log.warn("-- Done #{backend} ------------")
  end

  # ================================================================
  protected

  #
  # Assigns localID or overrides it
  # 
  def getLocalId(localID)
    cfg = baseConfig.get_parameters('/ICSystemConf/Host')
    if(defined?(ICAC_IDENT_OVER))
      @localID = ICAC_IDENT_OVER
    elsif(cfg && cfg['Host'])
      @localID = cfg['Host']
    elsif(!localID.nil?)
      @localID = localID
    elsif ENV['HOST']
      @localID = ENV['HOST']
    else
      @localID = Socket.gethostname		
    end
  end


  #
  # Get basic XML configuration for Bender
  # 
  def getBaseConfig(baseConfig='baseconfig.xml')

    xmlConfigFile = @configRoot + "/" + baseConfig
    #print("xmlConfFile: #{xmlConfigFile}\n")
    @baseConfig = ConfigFile.new(xmlConfigFile)
    @baseConfig.no_hash_key_element_path = true

  end


  #
  # starts the logging system
  # 
  def startLogger
    logDest = baseConfig['/ICSystemConf/Log/path']
    if(logDest.nil?)
      raise ICAutoConfigError('Base Configuration does not contain a log destination')
    end
    
    if(!$silent)
      $log = Logger.new(STDOUT)
    elsif(logDest == 'STDOUT')
      $log = Logger.new(STDOUT)
    else
      $log = Logger.new(logDest)
    end
    
    @log = Log.instance
    
    logLevel = baseConfig.get_int('/ICSystemConf/Log/level')

    if((!$silent) && $info)
      logLevel = Logger::Severity::INFO
    end
    if((!$silent) && $debug)
      logLevel = Logger::Severity::DEBUG
    end
      
    if((logLevel >= 0) && (logLevel <= 5))
      #print("level = #{logLevel} debug=#{Logger::DEBUG}\n")
      log.level = logLevel
    end	
    
    $log.formatter = ->(severity, time, progname, msg) {
      msg = case msg
      when ::String
        msg
      when ::Exception
        "#{ msg.message } (#{ msg.class })\n" <<
          (msg.backtrace || []).join("\n")
      else
        msg.inspect
      end

      "#{msg}\n"
    }

    log.debug("-- Logger started")
  end

  protected

  def bender_root
    File.dirname(File.dirname(__FILE__))
  end
end


# FIXME
# this reverses a monkey patch in active_support/clean_logger.rb:
# see http://www.ruby-forum.com/topic/93539

class Logger

	if($silent)
	if private_method_defined?(:old_format_message)
   		alias format_message old_format_message
	end
	end
end




# $URL: https://svn.intercom.it/svn/intercom/projects/Bender-NG/lib/classes/ICBackend.rb $
# $Rev: 1374 $
# $Author: lele $
# $Date: 2007-07-03 23:19:41 +0200 (Tue, 03 Jul 2007) $
# $Id: ICBackend.rb 1374 2007-07-03 21:19:41Z lele $
