require 'bender/classes/Plugin'
require 'securerandom'

$pluginClass = :MySQL

class MySQL < ICPlugin
  def initialize(*args)
    super

    conn = baseConfig.get_string_array('/ICSystemConf/VhostsDatabase')
    @mysql = Mysql2::Client.new(conn[0])
  end

  def provision(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    createMysqlDatabases(host, service, env)
  end

  def update(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    createMysqlDatabases(host, service, env)
  end

  def maintain(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]
  end

  def deactivate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    revokeAccessToDatabases(host, service, env)
  end

  def activate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    grantAccessToDatabases(host, service, env)
  end

  def prepare(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  def cleanup(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  protected

  def createMysqlDatabases(host, service, env)
    log.debug "  + provision databases"
    if host.databases && host.databases.any?
      active = host.databases.select{|d| d[:type] == 'mysql' && d[:enabled] }
      not_active = host.databases.select{|d| d[:type] == 'mysql' && !d[:enabled]}

      # master user for phpMyAdmin
      if active.any?
        exec_query "CREATE USER IF NOT EXISTS `%s`@'%%';" % @mysql.escape(host.service.login)
      end

      if active.any?
        active.each do | db |
          exec_query "CREATE DATABASE IF NOT EXISTS %s;" % @mysql.escape(db.database)
          
          # WARN
          # potrebbe essere che l'user del db sia uguale al master user
          # SKIP if db user is == master user
          unless db.login == host.service.login
            # database user
            exec_query "CREATE USER IF NOT EXISTS `%s`@'%%';" % @mysql.escape(db.login)
            exec_query "GRANT ALL PRIVILEGES ON `%s`.* to `%s`@'%%';" % [@mysql.escape(db.database), @mysql.escape(db.login)]

            # master user for phpMyAdmin
            exec_query "GRANT ALL PRIVILEGES ON `%s`.* to `%s`@'%%';" % [@mysql.escape(db.database), @mysql.escape(host.service.login)]
          end

          uri = "%s://%s:%s@%s:%d/%s" % [
            db.type,
            db.login,
            db.password,
            service.mySvcConf("/Services/Database[@name=\"#{db.type}\"]/Host"),
            service.mySvcConf("/Services/Database[@name=\"#{db.type}\"]/Port"),
            db.database
          ]
          env[:env_vars]['DATABASE_URL'] ||= uri
          env[:env_vars]["DATABASE_#{db.database}_URL".upcase] = uri
        end
      end

      if not_active.any?
        not_active.each do | db |
          
          # master user for phpMyAdmin
          exec_query "REVOKE ALL PRIVILEGES ON `%s`.* FROM %s@'%%';" % [@mysql.escape(db.database), @mysql.escape(host.service.login)]
          
          # WARN
          # potrebbe essere che l'user del db sia uguale al master user
          # SKIP if db user is == master user
          unless db.login == host.service.login
            # database user
            exec_query "REVOKE ALL PRIVILEGES ON `%s`.* FROM %s@'%%';" % [@mysql.escape(db.database), @mysql.escape(db.login)]

            exec_query "DROP USER IF EXISTS %s@'%%';" % @mysql.escape(db.login)
          end
        end
      end

      # master user for phpMyAdmin
      unless active.any?
        exec_query "DROP USER IF EXISTS %s@'%%';" % @mysql.escape(host.service.login)
      end

      exec_query "FLUSH PRIVILEGES;"
    end
  end

  def revokeAccessToDatabases(host, service, env)
    log.debug "  + disable database access"
    if host.databases && host.databases.any?
      # master user
      set_password(host.service.login, SecureRandom.hex)
      
      host.databases.each do | db |
        next unless db[:type] == 'mysql'
        # WARN
        # potrebbe essere che l'user del db sia uguale al master user
        # SKIP if db user is == master user
        next if db.login == host.service.login
        set_password(db.login, SecureRandom.hex)
      end
    end
  end

  def grantAccessToDatabases(host, service, env)
    log.debug "  + enable database access"
    if host.databases && host.databases.any?
      active = host.databases.select{|d| d[:type] == 'mysql' && d[:enabled] }
      not_active = host.databases.select{|d| d[:type] == 'mysql' && !d[:enabled]}

      # master user
      if active.any?
        set_password(host.service.login, host.service.password)
      else
        set_password(host.service.login, SecureRandom.hex)
      end

      active.each do | db |
        # WARN
        # potrebbe essere che l'user del db sia uguale al master user
        # SKIP if db user is == master user
        next if db.login == host.service.login
        set_password(db.login, db.password)
      end

      not_active.each do | db |
        # WARN
        # potrebbe essere che l'user del db sia uguale al master user
        # SKIP if db user is == master user
        next if db.login == host.service.login
        set_password(db.login, SecureRandom.hex)
      end
    end
  end

  protected
  def set_password(login, password)
    q = "SELECT count(*) AS `exists` FROM mysql.user WHERE user = '%s'" % @mysql.escape(login)
    res = @mysql.query(q)
    if res.first && res.first['exists'] > 0
      q = "ALTER USER '%s' IDENTIFIED BY '%s';" % [ @mysql.escape(login), '********' ]
      log.debug "  + #{q}"
      q = "ALTER USER '%s' IDENTIFIED BY '%s';" % [ @mysql.escape(login), @mysql.escape(password) ]
      res = @mysql.query(q)
    end
  end

  def exec_query(q, fail = false)
    log.debug "  + #{q}"
    res = @mysql.query(q)
  rescue Mysql2::Error => ex
    log.error "    # #{ex.to_s}"
    raise ex if fail
  end
end
