require 'bender/classes/Plugin'

$pluginClass = :PHPFPM

class PHPFPM < ICPlugin
  def provision(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    return unless enabled?(host)

    PHPFPMConfig(host, service, env)
    PHPFPMSysconfig(host, service, env)
  end

  def update(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    return unless enabled?(host)

    PHPFPMConfig(host, service, env)
    PHPFPMSysconfig(host, service, env)
  end

  def maintain(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    return unless enabled?(host)
  end

  def deactivate(args)
    service = args[:service]
    env = args[:env]
  end

  def activate(args)
    service = args[:service]
    env = args[:env]
  end

  def prepare(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  def cleanup(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  protected

  def PHPFPMConfig(host, service, env)
    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')

    data = {}

    data[:username] = service.containerUsername(host)

    vhostroot = service.containerVHostRoot(host)

    data[:socket_path] = File.join(vhostroot, 'php-fpm.sock')

    _instances = host.php.instances
    _memory_limit = host.php.memory_limit
    _upload_max_filesize = host.php.upload_max_filesize

    _quota_memory = nil
    _quota_memory_hard = nil
    if host.quota.quota_name.present?
      _quota_memory = service.mySvcConf("/Quotas/Quota[@name='#{host.quota.quota_name}']/Memory").to_i
      _quota_memory_hard = service.mySvcConf("/Quotas/Quota[@name='#{host.quota.quota_name}']/MemoryHard").to_i
    else
      _quota_memory = host.quota.quotas.mem.try(:to_i)
      _quota_memory_hard = _quota_memory + 256 if _quota_memory
    end
    _quota_memory ||= 256
    _quota_memory_hard ||= 4096

    _memory_limit = service.mySvcConf('/ConfigDefaults/PHP/MemoryLimit') if _memory_limit.blank?
    _upload_max_filesize = service.mySvcConf('/ConfigDefaults/PHP/UploadMaxFilesize') if _upload_max_filesize.blank?
    _memory_limit = [_memory_limit.to_i, _upload_max_filesize.to_i].max

    _instances = _instances.to_i if _instances.to_s =~ /\A[0-9]/
    _memory_limit = _memory_limit.to_i if _memory_limit.to_s =~ /\A[0-9]/
    _upload_max_filesize = _upload_max_filesize.to_i if _upload_max_filesize.to_s =~ /\A[0-9]/

    if _instances == 'AUTO'
      _instances = (_quota_memory.to_i / _memory_limit.to_i).floor
    end

    # limit php max (75%|mem-256) of allocated memory
    _maxmem = [(_quota_memory_hard*0.75).ceil, (_quota_memory_hard*-256)].max
    while _instances * _memory_limit > _maxmem
      _instances -= 1
      _instances = 1 if _instances <= 0
      _memory_limit = [_memory_limit, _maxmem].min
    end

    # limit upload_max_filesize to memory limit
    _upload_max_filesize = [_upload_max_filesize.to_i, _memory_limit.to_i].min

    data[:max_children] = _instances
    data[:start_servers] = 1
    data[:min_spare_servers] = 1
    data[:max_spare_servers]= [3, data[:max_children] - data[:min_spare_servers]].min
    data[:max_spare_servers]= [1, data[:max_spare_servers]].max
    data[:memory_limit] = _memory_limit
    env[:settings][:max_body_size] = data[:max_post_size] = _upload_max_filesize
    data[:upload_max_filesize] = _upload_max_filesize

    data[:additional] = host.php.additional.lines.map(&:strip)

    data[:email_envelope_address] = "%s@%s" % [
      host.smtp.try(:username) || host.service.login,
      host.smtp.try(:domain) || service.mySvcConf('/Services/SMTP/Domain')
    ]

    data[:host] = host

    template = File.join(bender_root, 'templates', service.serviceName, "php-fpm.erb")

    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='PHPFPM2Vhosts']/path"),
      "#{host.service.name}.conf")

    ctFname = File.join(service.mySvcConf("/ContainerPaths/PHPFPMConfigPath"),
      "#{host.service.name}.conf")

    env[:configs][:php_fpm_conf] = {host: fname, container: ctFname}

    out = service.render(template, data)
    service.write(fname, out)
  end

  def PHPFPMSysconfig(host, service, env)
    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')

    phpver = env[:settings][:php_version] = phpVersion(host, service)

    data = {}
    data[:extensions] = service.mySvcConf("/ConfigDefaults/PHP/RequiredModules[@name='#{phpver}']").split(',').map(&:strip)
    data[:extensions] += host.php.extensions
    data[:extensions] = data[:extensions].sort.uniq

    data[:host] = host

    template = File.join(bender_root, 'templates', service.serviceName, "php-fpm_sysconfig.erb")

    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='PHPFPM2Sysconfig']/path"),
      "#{host.service.name}.conf")

    ctFname = File.join(service.mySvcConf("/ContainerPaths/SysconfigPath"),
      service.mySvcConf("/ContainerPaths/PHPFPMSysconfigFile"))

    env[:configs][:php_fpm_syscfg] = {host: fname, container: ctFname}

    out = service.render(template, data)
    service.write(fname, out)
  end

  def enabled?(host)
    host.php.enabled
  end

  def phpVersion(host, service)
    phpver = host.php.version.strip
    phpver = 'latest' if phpver.blank?
    phpver = case phpver
    when 'default' then service.mySvcConf('/ConfigDefaults/PHP/DefaultVersion')
    when 'latest'  then service.mySvcConf('/ConfigDefaults/PHP/LatestVersion')
    else phpver
    end
  end
end
