require 'bender/classes/Plugin'

$pluginClass = :Apache2

class Apache2 < ICPlugin
  def provision(args)
    host = args[:host]
    service = args[:service]
    env = args[:env] 

    return unless enabled?(host)

    apache2Config(host, service, env)
    apache2Sysconfig(host, service, env)
  end

  def update(args)
    host = args[:host]
    service = args[:service]
    env = args[:env] 

    return unless enabled?(host)

    apache2Config(host, service, env)
    apache2Sysconfig(host, service, env)
  end

  def maintain(args)
    host = args[:host]
    service = args[:service]
    env = args[:env] 
    
    return unless enabled?(host)
  end

  def deactivate(args)
    service = args[:service]
    env = args[:env] 
  end

  def activate(args)
    service = args[:service]
    env = args[:env] 
  end

  def prepare(args)
    service = args[:service]
    action = args[:action]
    env = args[:env] 
  end

  def cleanup(args)
    service = args[:service]
    action = args[:action]
    env = args[:env] 
  end

  protected

  def apache2Config(host, service, env)
    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')
    
    template = 'apache24'           
    if host.http.apache2.compat
      template += '-compat'
    end

    template = File.join(bender_root, 'templates', service.serviceName, "#{template}.erb")

    data = {}
    data[:vhost_webmaster] = service.mySvcConf('/ConfigDefaults/WebMaster') + '@' + host.service.servername

    data[:vhostroot] = service.containerVHostRoot(host)

    data[:docroot] = unless host.http.apache2.docroot.blank?
      File.join(data[:vhostroot], host.http.apache2.docroot.strip)
    else
      File.join(data[:vhostroot], service.mySvcConf('/ServicePaths/UserSpace/DocRootPath'))
    end

    data[:options] = unless host.http.apache2.global_options.blank?
      host.http.apache2.global_options
    else
      service.mySvcConf('/ConfigDefaults/Options')
    end

    data[:override] = unless host.http.apache2.allow_override.blank?
      host.http.apache2.allow_override
    else
      service.mySvcConf('/ConfigDefaults/AllowOverride')
    end

    # cake needs fileinfo ability

    if host.php && host.php.legacy.cakephp
      if(data[:options].downcase.include?('none'))
        data[:options] = ''
      end
      if(data[:override].downcase.include?('none'))
        data[:override] = ''
      end

      if(!data[:options].downcase.include?('followsymlinks'))
        data[:options] += ' +FollowSymLinks'
      end
      if(!data[:override].downcase.include?('fileinfo'))
        data[:override] += ' FileInfo'
      end
    end

    data[:stats_location] = service.mySvcConf('/ServicePaths/UserSpace/StatsPath')
    data[:stats_path] = File.join(data[:vhostroot], service.mySvcConf('/ServicePaths/UserSpace/StatsPath'))

    if host.php.enabled
      data[:userid] = service.containerUID(host)
      data[:username] = service.containerUsername(host)

      data[:socket_path] = File.join(data[:vhostroot], 'php-fpm.sock')
    end
    
    data[:additional] = (host.http.apache2.additional||'').lines.map(&:strip)
    data[:env] = env
    data[:host] = host
    
    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='Apache2Vhosts']/path"),
      "#{host.service.name}.conf")
    
    ctFname = File.join(service.mySvcConf("/ContainerPaths/Apache2ConfigPath"),
      "#{host.service.name}.conf")

    env[:configs][:apache2_conf] = {host: fname, container: ctFname}

    out = service.render(template, data)
    service.write(fname, out)
  end

  def apache2Sysconfig(host, service, env)
    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')
    
    data = {}
    data[:apache2_modules] = service.mySvcConf("/ConfigDefaults/Apache2/RequiredModules").split(',').map(&:strip)
    data[:apache2_modules] += host.http.apache2.modules
    data[:apache2_modules] << 'access_compat' if host.http.apache2.compat
    data[:apache2_modules] = data[:apache2_modules].sort.uniq

    data[:host] = host

    template = File.join(bender_root, 'templates', service.serviceName, "apache2_sysconfig.erb")

    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='Apache2Sysconfig']/path"),
      "#{host.service.name}.conf")

    ctFname = File.join(service.mySvcConf("/ContainerPaths/SysconfigPath"),
      service.mySvcConf("/ContainerPaths/Apache2SysconfigFile"))

    env[:configs][:apache2_syscfg] = {host: fname, container: ctFname}

    out = service.render(template, data)
    service.write(fname, out)
  end

  def parseEnv(txt)
    txt ||= ''
    env =  {}
    begin
      env = YAML.load(txt)
    rescue
      begin
        env = JSON.parse(txt)
      rescue
        env = txt.lines.map(&:strip).reject(&:blank?)
        env = env.map{|e| e.split('=', 2).map(&:strip)}
        env = Hash[env]
      end
    end
    env || {}
  end

  def enabled?(host)
    host.http.enabled
  end
end
