require 'bender/classes/Plugin'
require 'fileutils'

$pluginClass = :Proxy

class Proxy < ICPlugin
  def initialize(*args)
    super
    @restart = false
  end

  def provision(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    setupProxy(host, service, env)
  end

  def update(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    setupProxy(host, service, env)
  end

  def maintain(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    return unless enabled?(host)
  end

  def deactivate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]
  end

  def activate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]
  end

  def prepare(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  def cleanup(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  protected

  def setupProxy(host, service, env)
    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')

    proxyRoot = File.join(basePath, service.mySvcConf('/System/ProxyRoot'),
      host.service.name, service.mySvcConf('/ContainerPaths/ProxyWebRoot'))

    webroot = File.join(service.mySvcConf('/ContainerPaths/ProxyRoot'),
      host.service.name, service.mySvcConf('/ContainerPaths/ProxyWebRoot'))

    maintenance_filename  = "%s_maintenance.html" % Digest::MD5.hexdigest(webroot + '/maintenance.html')
    unauthorized_filename = "%s_403.html" % Digest::MD5.hexdigest(webroot + '/403.html')
    servererror_filename  = "%s_500.html" % Digest::MD5.hexdigest(webroot + '/500.html')

    maintenance_tpl  = File.join(bender_root, 'templates', service.serviceName, "maintenance.html")
    unauthorized_tpl = File.join(bender_root, 'templates', service.serviceName, "403.html")
    servererror_tpl  = File.join(bender_root, 'templates', service.serviceName, "500.html")

    env[:settings][:proxy] = {
      webroot: webroot,
      maintenance_filename: maintenance_filename,
      unauthorized_filename: unauthorized_filename,
      servererror_filename: servererror_filename,
      upstream: "www-#{host.service.login}".dasherize,
      upstream_port: service.httpPort(host),
      auto_route: [host.service.login.dasherize, service.mySvcConf('/ConfigDefaults/DefaultRouteBase')].join('.'),
      servername: host.service.servername,
      aliases: host.service.aliases,
      access_log: File.join(service.mySvcConf('/ContainerPaths/ProxyLogRoot'), host.service.name,
        service.mySvcConf('/ConfigDefaults/AccessLog'))
    }
    env[:settings][:max_body_size] = [ 
      host.proxy.max_body_size || 0, 
      env[:settings][:max_body_size] || 0,
      service.mySvcConf('/ConfigDefaults/DefaultBodySize').to_i
    ].max

    if enabled?(host)
      service.write(File.join(proxyRoot, maintenance_filename),  File.read(maintenance_tpl))
      service.write(File.join(proxyRoot, unauthorized_filename), File.read(unauthorized_tpl))
      service.write(File.join(proxyRoot, servererror_filename),  File.read(servererror_tpl))
    end
  end

  def enabled?(host)
    host.service.active
  end
end
