require 'bender/classes/Plugin'
require 'bender/tools/Templates'
require 'fileutils'

$pluginClass = :ICLogRotate


class ICLogRotate < ICPlugin
  def rotate(env)
    logs = ['ErrorLog', 'AccessLog']
    months = {  'Jan' => '01',
          'Feb' => '02',
          'Mar' => '03',
          'Apr' => '04',
          'May' => '05',
          'Jun' => '06',
          'Jul' => '07',
          'Aug' => '08',
          'Sep' => '09',
          'Oct' => '10',
          'Nov' => '11',
          'Dec' => '12'}

    host = env[:host]
    service = env[:service]

    basePath =  baseConfig.get_entry('/ICSystemConf/BaseDir')
    loghome = service.host_logPath(host)

    arcloghome =  loghome + '/' + service.mySvcConf('/Logs/LogArchivePath') + '/'


    logs.each { |thisLog|
      logname = service.mySvcConf("/ConfigDefaults/#{thisLog}")
      filename = loghome + '/' + logname

      if(thisLog == 'ErrorLog')
        regex = Regexp.compile('^\[.{3} (.{3}) (.{2}) .{15} (.{4})\].*$')
      else
        regex = Regexp.compile('^.*?\[(.{2})\/(.{3})\/(.{4}).*$')
      end

      log.debug(" + processing log #{filename}")
      next unless(FileTest::exists?(filename))

      FileUtils.cp(filename, filename + '.tmp')

      #truncate
      File.open(filename, "w+") {}

      ## dispatch lines in the appropriate log

      File.open(filename + '.tmp', 'r') { |file |

        file.each_line { |line |
          match = regex.match(line)
          if(thisLog == 'ErrorLog')
            outfile = "#{logname}-#{match[3]}#{months[match[1]]}#{match[2]}"
          else
            outfile = "#{logname}-#{match[3]}#{months[match[2]]}#{match[1]}"
          end
          open(arcloghome + '/' + outfile, 'a') { |f| f.puts line }
        }
      }
    }

    # skip today and yesterday logs
    time = Time.new.localtime.at_midnight
    today = time.strftime("%Y%m%d")
    yesterday = (time-1.days).strftime("%Y%m%d")

    logs.each { |thisLog|
      logname = service.mySvcConf("/ConfigDefaults/#{thisLog}")
      filename = arcloghome + '/' + logname
      skip = ["#{filename}-#{today}", "#{filename}-#{yesterday}"]

      Dir["#{filename}-[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]"].each do | f |
        # skip today and yesterday logs
        next if skip.include?(f)
        log.debug " + compressing #{f}"

        orig = f
        dest = "#{f}.gz"
        c = 0
        # duplicate file?
        ko = File::exists?(dest)
        while ko
          c += 1
          dest = "#{f}-#{c}.gz"
          # never never overwrite logs
          ko = File::exists?(dest) || File::exists?(orig)
          if ko
            orig = "#{f}-#{c}"
          end
        end
        if orig != f
          log.debug " + rename #{f} -> #{orig}"
          FileUtils::mv(f, orig)
        end
        service.run("gzip \"#{orig}\"")
      end
    }
  end

  def provision(env)
  end

  def update(env)
  end

  def maintain(env)
    rotate(env)
  end
end
