require 'bender/classes/Plugin'
require 'securerandom'

$pluginClass = :Docker

class Docker < ICPlugin
  def provision(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    dockerCompose(host, service, env)
  end

  def update(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    dockerCompose(host, service, env)
  end

  def maintain(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]
  end

  def deactivate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    disableStack(host, service, env)
  end

  def activate(args)
    host = args[:host]
    service = args[:service]
    env = args[:env]

    enableStack(host, service, env)
  end

  def prepare(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  def cleanup(args)
    service = args[:service]
    action = args[:action]
    env = args[:env]
  end

  protected

  def dockerCompose(host, service, env)
    docker_image = if host.service.docker_image.present?
      host.service.docker_image
    elsif env[:settings][:php_version].present?
      service.mySvcConf("/Docker/Images/Image[@name=\"#{env[:settings][:php_version]}\"]/Name")
    else
      service.mySvcConf("/Docker/Images/Default/Name")
    end

    overbooking = service.mySvcConf("/QuotaOverbooking").to_i

    quota_memory = nil
    quota_memory_hard = nil

    if host.quota.quota_name
      quota_memory = service.mySvcConf("/Quotas/Quota[@name='#{host.quota.quota_name}']/Memory").to_i
      quota_memory_hard = service.mySvcConf("/Quotas/Quota[@name='#{host.quota.quota_name}']/MemoryHard").to_i
    else
      quota_memory = host.quota.quotas.mem.try(:to_i)
      quota_memory_hard = quota_memory + 256 if quota_memory
    end

    if quota_memory && overbooking && overbooking > 0
      quota_memory -= quota_memory*overbooking/100
    end

    unless quota_memory
      quota_memory = 32
      quota_memory_hard = 4096
    end

    networks = service.serviceConfig.get_string_array(service.mySvcConfPath('/Docker/Compose/Networks/Network')).map{|a| a['Network']}

    env_vars = env[:env_vars].map do |k,v|
      v = v.to_s.gsub(/\$/, '$$')
      "%s=%s" % [k,v]
    end

    random = SecureRandom.alphanumeric(8)

    volumes = {}
    mounts = []
    env[:volumes].map do | k, v |
      name = "#{k}-#{random}"
      cfg = {
        'driver' => 'local',
        'driver_opts' => {
          'device' => v[:volume][:device],
          'o' => v[:volume][:options],
          'type' => v[:volume][:type]
        }
      }

      volumes[name] = cfg
      mounts << {
        'type' => 'volume',
        'source' => name,
        'target' => v[:mount],
        'read_only' => v[:read_only]
      }
    end

    configs = {}
    configfiles = []
    env[:configs].map do | k, v |
      configs[k.to_s] = { 'file' => v[:host] }
      configfiles << { 'source' => k.to_s, 'target' => v[:container] }
    end

    labels = {
      'vhosts-ng/service/name' => host.service.name,
      'vhosts-ng/service/auto-route' => env[:settings][:proxy][:auto_route],
      'vhosts-ng/service/server-name' => env[:settings][:proxy][:servername],
      'vhosts-ng/service/upstream' => env[:settings][:proxy][:upstream],
      'vhosts-ng/service/upstream-port' => env[:settings][:proxy][:upstream_port],
      'vhosts-ng/service/access-log' => env[:settings][:proxy][:access_log],
      'vhosts-ng/http' => (!host.ssl.enabled || host.ssl.allow_http).to_s,
      'vhosts-ng/https' => (host.ssl.enabled).to_s,
      'vhosts-ng/letsencrypt' => (host.ssl.letsencrypt).to_s,
      'vhosts-ng/html/webroot' => env[:settings][:proxy][:webroot],
      'vhosts-ng/html/maintenance' => env[:settings][:proxy][:maintenance_filename],
      'vhosts-ng/html/40x' => env[:settings][:proxy][:unauthorized_filename],
      'vhosts-ng/html/50x' => env[:settings][:proxy][:servererror_filename],
    }
    if env[:settings][:proxy][:aliases].try(&:any?)
      labels['vhosts-ng/service/aliases'] = env[:settings][:proxy][:aliases].join(',')
    end
    if host.ssl.enabled

      if host.ssl.letsencrypt
        labels['vhosts-ng/ssl/letsencrypt_name'] = 'letsencrypt'
      end

      if env[:settings][:ssl][:certs] && env[:settings][:ssl][:certs].any?
        env[:settings][:ssl][:certs].each do |k,v|
          labels["vhosts-ng/ssl/certs/#{k}"] = v
        end
      end
    end
    if env[:settings][:max_body_size]
      labels['vhosts-ng/max_body_size'] = "%dm" % env[:settings][:max_body_size]
    end

    config = {
      'version' => "3.7",
      'services' => {
        'www' => {
          'image' => docker_image,
          'hostname' => env[:settings][:proxy][:upstream],
          'networks' => Hash[networks.map{|n|
            [n, {'aliases' => [env[:settings][:proxy][:upstream]]}]
          }],
          'deploy' => {
            'labels' => labels,
            'placement' => {
              #'constraints' => [
              #  "node.role == worker"
              #]
            },
            'resources' => {
              'limits' => {
                'memory' => "#{quota_memory_hard}m"
              },
              'reservations' => {
                'memory' => "#{quota_memory}m"
              }
            },
            'restart_policy' => {
              'condition' => 'any',
              'delay' => '10s',
              'max_attempts' => 3,
              'window' => '30s'
            }
          },
          'configs' => configfiles,
          'volumes' => mounts,
          'environment' => env_vars
        }
      },
      'networks' => Hash[networks.map{|n|
        [n, {'external' => true, 'name' => n}]
      }],
      'volumes' => volumes,
      'configs' => configs
    }

    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')

    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='DockerComposeDir']/path"),
      "vhost-#{host.service.login}.yml")

    service.write(fname, YAML.dump(config))
  end

  def disableStack(host, service, env)
    log.debug '  + disable docker stack'

    service.run "docker service scale vhost-#{host.service.login}_www=0" if running?(host)
    service.run "docker stack rm vhost-#{host.service.login}"
  end

  def enableStack(host, service, env)
    log.debug '  + enable docker stack'

    basePath = service.baseConfig.get_entry('/ICSystemConf/BaseDir')

    fname = File.join(basePath, service.mySvcConf("/MandatoryPaths/Path[@name='DockerComposeDir']/path"),
    "vhost-#{host.service.login}.yml")

    if env[:dirty]
      service.run "docker service scale vhost-#{host.service.login}_www=0" if exists?(host) && running?(host)
      service.run "docker stack rm vhost-#{host.service.login}" if exists?(host) && env[:dirty]
      service.run "docker stack deploy --with-registry-auth --compose-file \"#{fname}\" vhost-#{host.service.login}"
    else
      # force update the stack
      service.run "docker service update --force vhost-#{host.service.login}_www"
    end
  end

  def running?(host)
    `docker stack ps vhost-#{host.service.login} > /dev/null 2>&1`
    $?.success?
  end

  def exists?(host)
    `docker stack services vhost-#{host.service.login} > /dev/null 2>&1`
    $?.success?
  end
end
