#
# Project
#
# Copyright (C) 2007  Windmill Media
# Copyright (c) 2006-2007 Lele Forzani <lele@windmill.it>
#
#
# = filename.rb
#
# <short descr>
#
# Author:: Lele Forzani <lele@windmill.it>
# Documentation::  Lele Forzani <lele@windmill.it>
# License::
#   This is NOT free software. License for using this software is granted
#   by written permission from Lele Forzani <lele@windmill.it>
# Revision:: $Id: Webserver.rb 1742 2007-10-04 09:58:09Z lele $
#
#
# == Description
#
#
# == Annotations
#
#
# == Todo



require 'bender/classes/ICServiceBackend'
require 'bender/classes/Models/Webserver/WebHosts'

require 'bender/tools/ErbBindable'
require 'erb'

class ICWebserver < ICServiceBackend
  SERVICE = 'webserver'

  def initialize(localID, configRoot)
    @serviceName = SERVICE
    super
    Webserver::WebHost.open(baseConfig.get_string('/ICSystemConf/DatabasePath'))
  end
  #
  # Iterator over each host
  #
  def iterateHosts(args = {}, options = {}, &proc)
    active = options[:active]
    webHosts = []
    if active == true
      webHosts = Webserver::WebHost.find_all_active if args[:service].blank?
      webHosts = [Webserver::WebHost.find_active_by_username(args[:service])].compact unless args[:service].blank?
    elsif active == false
      webHosts = Webserver::WebHost.find_all_not_active if args[:service].blank?
      webHosts = [Webserver::WebHost.find_not_active_by_username(args[:service])].compact unless args[:service].blank?
    else
      webHosts = Webserver::WebHost.find_all if args[:service].blank?
      webHosts = [Webserver::WebHost.find_by_username(args[:service])].compact unless args[:service].blank?
    end

    webHosts.each do | h |
      proc.call(h.data)
    end
  end

  #
  # Main entries
  #
  def provisionService(args = {})
    return unless super

    env = {}
    execPlugins('webserver', 'prepare', {:service => self, :action => :provision, env: env})
    iterateHosts(args, active: true) do |h|
      env[:configs] = {}
      env[:settings] = {}
      env[:env_vars] = (h.service.environment||{}).merge(
        'USER_ID' => containerUID(h),
        'USER_NAME' => containerUsername(h),
        'USER_HOME' => containerVHostRoot(h),
        'PORT' => httpPort(h)
      )
      env[:volumes] = {}

      env[:dirty] = false
      log.info("- #{h.service.servername}")
      #checkHostConfiguredUsers(h)
      checkHostConfiguredPaths(h)
      checkHostWritablePaths(h)

      if(checkHostDirty(h, args))
        env[:dirty] = true
        #checkHostConfiguredUsers(h, true)
        execPlugins('webserver', 'provision', {:host => h, :service => self, env: env})
        touchHost(h)
      end

      execPlugins('webserver', 'activate', {:host => h, :service => self, env: env})
    end

    execPlugins('webserver', 'cleanup', {:service => self, :action => :provision, env: env})
  end

  def updateService(args = {})
    return unless super

    env = {}
    execPlugins('webserver', 'prepare', {:service => self, :action => :update, env: env})

    iterateHosts(args, active: true) do |h|
      log.info("- #{h.service.servername}")
      env[:configs] = {}
      env[:settings] = {}
      env[:env_vars] = (h.service.environment||{}).merge(
        'USER_ID' => containerUID(h),
        'USER_NAME' => containerUsername(h),
        'USER_HOME' => containerVHostRoot(h),
        'PORT' => httpPort(h)
      )
      env[:volumes] = {}

      env[:dirty] = false
      log.info("- #{h.service.servername}")
      #checkHostConfiguredUsers(h)
      checkHostConfiguredPaths(h)
      checkHostWritablePaths(h)

      if(checkHostDirty(h, args))
        env[:dirty] = true
        checkHostConfiguredUsers(h, true)
        execPlugins('webserver', 'provision', {:host => h, :service => self, env: env})
        touchHost(h)
      end

      execPlugins('webserver', 'activate', {:host => h, :service => self, env: env})
    end
    
    execPlugins('webserver', 'cleanup', {:service => self, :action => :update, env: env})
  end
  
  def deprovisionService(args = {})
    env = {}
    iterateHosts(args, active: false) do |h|
      log.info("- #{h.service.servername}")
      env[:configs] = {}
      env[:settings] = {}
      env[:env_vars] = (h.service.environment||{}).merge(
        'USER_ID' => containerUID(h),
        'USER_NAME' => containerUsername(h),
        'USER_HOME' => containerVHostRoot(h),
        'PORT' => httpPort(h)
      )
      env[:volumes] = {}
      execPlugins('webserver', 'deactivate', {:host => h, :service => self, env: env})
      removeTimestamp(h)
    end
  end

  def maintainService(args = {})
    return unless super
    execPlugins('webserver', 'prepare', {:service => self, :action => :maintain})

    iterateHosts(args, active: true) { |h|
      if h.service.active
        log.info("- #{h.service.servername}")
        checkHostWritablePaths(h)
        execPlugins('webserver', 'maintain', {:host => h, :service => self})
      end
    }

    execPlugins('webserver', 'cleanup', {:service => self, :action => :maintain})
  end

  # ======================================================================

  private

  #
  # This host needs rebuilding?
  #

  def checkHostDirty(host, args = {})
    storagePath = host_storagePath(host)
    begin
      timestamp = Time.parse(IO.read(storagePath + ".ts"))
    rescue
      timestamp = Time.at(0)
    end
    timestamp = Time.at(0) if args[:force]

    if FileTest::exists?(storagePath + ".aclck")
      log.info("  + Host '#{host.service.login}' is manually locked")
      return false
    end

    #log.debug("  +++ database ts=#{host.timestamp}, lock ts=#{timestamp}")

    if host.service.updated_at > timestamp
      log.info("  + Host '#{host.service.login}' must be rebuilt")
      return true
    end

    return false
  end

  #
  # Checks if system users exist for this host
  #

  #def checkHostConfiguredUsers(host, changepw = false)
  #  user = host.service.login
  #  pass = host.service.password
  #  id   = svcUID(host)
#
  #  # sanity checks
  #  if user.blank?
  #    raise Exceprion.new("Host with empty login?")
  #  end
#
  #  # does this user exist
  #  begin
  #    usr = Etc.getpwnam(user)
  #  rescue ArgumentError
  #    # user doesn't exist
#
  #    gid = mySvcConf('/SystemUsers/DefaultGroupId').to_i
  #    path = host_storagePath(host)
  #    shell = mySvcConf('/SystemUsers/DefaultShell')
#
  #    mkuser = `useradd -d #{path} -s #{shell} -g #{gid} -u #{id}  #{user}`
  #    log.debug(mkuser)
  #  end
#
  #  begin
  #    usr = Etc.getpwnam(user)
  #  rescue ArgumentError
  #    # something went wrong
  #    log.error("CheckUsers: something is wrong with use #{user}, unable to add")
  #    return
  #  end
#
  #  if usr.uid != id
  #    log.error("CheckUsers: something is wrong with use #{user}, id is mismatching with the system")
  #  end
  #end

  # Checks if hosts have the necessary writable paths
  # and they have appropriate permissions
  #

  def checkHostWritablePaths(host)
    storagePath = host_storagePath(host)
    docRoot = File.join(storagePath, mySvcConf('/ServicePaths/UserSpace/DocRootPath'))

    uid = svcUID(host)
    gid = mySvcConf('/WritablePaths/gid')
    mode = mySvcConf('/WritablePaths/mode')

    return unless host.php.enabled && host.php.legacy.writable_storage

    host.php.legacy.writable_storage.each do | path |

      writablepath = File.join(docRoot, path.strip)

      begin
        SystemInterface.verifyPath(writablepath, true, mode, uid, gid)
      rescue SystemCallError => e
        log.error("checkHostWritablePaths: error in adding path '#{path}: #{e.message}'")
      end
    end
  end

  #
  # Checks if hosts have the necessary directories
  # and they have appropriate permissions
  #

  def checkHostConfiguredPaths(host)
    storagePath = host_storagePath(host)
    sslPath = host_sslPath(host)
    logPath = host_logPath(host)
    proxyPath = host_proxyPath(host)
    defaultMode = Integer(mySvcConf('/ServicePaths/DefaultMode'))

    uid = svcUID(host)
    gid = mySvcConf('/SystemUsers/DefaultGroup')

    begin
      SystemInterface.verifyPath(storagePath, true, defaultMode, uid, gid)
    rescue SystemCallError => e
      log.error("checkHostConfiguredPath: error in adding userpath '#{storagePath}: #{e.message}'")
    end

    begin
      SystemInterface.verifyPath(sslPath, true, defaultMode, 'root', 'root')
    rescue SystemCallError => e
      log.error("checkHostConfiguredPath: error in adding userpath '#{sslPath}: #{e.message}'")
    end

    begin
      SystemInterface.verifyPath(proxyPath, true, defaultMode, 'root', 'root')
    rescue SystemCallError => e
      log.error("checkHostConfiguredPath: error in adding userpath '#{sslPath}: #{e.message}'")
    end

    paths = serviceConfig.get_parameters(mySvcConfPath("ServicePaths/UserSpace/*"))

    paths.each do |pathName, pathEntry |
      path = File::join(storagePath, pathEntry)

      begin
        SystemInterface.verifyPath(path, true, defaultMode, uid, gid)
      rescue SystemCallError => e
        log.error("checkHostConfiguredPaths: error in adding path '#{path}: #{e.message}'")
      end
    end

    paths = serviceConfig.get_parameters(mySvcConfPath("ServicePaths/Logs/*"))

    paths.each do |pathName, pathEntry |
      path = File.join(logPath, pathEntry)

      begin
        SystemInterface.verifyPath(path, true, defaultMode, uid, gid)
      rescue SystemCallError => e
        log.error("checkHostConfiguredPaths: error in adding path '#{path}: #{e.message}'")
      end
    end
  end

  #
  # Updates timestamps for the host
  #

  def touchHost(host)
    storagePath = host_storagePath(host)

    File.open(storagePath + ".ts", "w+") do |f|
      f.write(host.service.updated_at)
    end
  end
  
  def removeTimestamp(host)
    storagePath = host_storagePath(host)

    File.delete(storagePath + ".ts") if File.exist?(storagePath + ".ts")
  end

  public

  # shortcut accessor to the home path
  def host_storagePath(host)
    basePath = baseConfig.get_entry('/ICSystemConf/BaseDir')
    path = File.join(basePath, mySvcConf('/System/StoragePath'),
      mySvcConf('/System/VhostsPath'), host.service.name)

    return path
  end

  def host_srvPath(host)
    basePath = baseConfig.get_entry('/ICSystemConf/BaseDir')
    path = File.join(basePath, mySvcConf('/System/SrvPath'),
      mySvcConf('/System/VhostsPath'), host.service.name)

    return path
  end

  def host_sslPath(host)
    basePath = baseConfig.get_entry('/ICSystemConf/BaseDir')
    path = File.join(basePath, mySvcConf('/System/CertsRoot'),
      host.service.name)

    return path
  end

  def host_logPath(host)
    basePath = baseConfig.get_entry('/ICSystemConf/BaseDir')
    path = File.join(basePath, mySvcConf('/System/StoragePath'),
      mySvcConf('/System/LogPath'), host.service.name)

    return path
  end

  def host_proxyPath(host)
    basePath = baseConfig.get_entry('/ICSystemConf/BaseDir')
    path = File.join(basePath, mySvcConf('/System/ProxyRoot'),
      host.service.name, mySvcConf('/ContainerPaths/ProxyWebRoot'))
    return path
  end

  def svcUID(host)
    #uid = mySvcConf('/SystemUsers/UIDOffset').to_i + host.service.id.to_i
    mySvcConf('/SystemUsers/DefaultContainerUID').to_i
  end

  def containerUID(host)
    svcUID(host)
  end

  def containerUsername(host)
    mySvcConf('/SystemUsers/DefaultContainerUser')
  end

  def containerVHostRoot(host)
    if host.service.legacy
      File.join(mySvcConf('/ContainerPaths/LegacyVhostRoot'), host.service.servername)
    else
      File.join(mySvcConf('/ContainerPaths/VhostRoot'))
    end
  end

  def render(template, data = {})
    raise Exception.new("Template not found #{template}") unless File.exists?(template)
    erb = ERB.new(File.read(template), nil, '-')
    bindable = ErbBindable.new(data || {})
    erb.result bindable.get_binding
  end

  def httpPort(host)
    if host.service.docker_image.present?
      mySvcConf('ConfigDefaults/UpstreamBasePort').to_i + host.service.id
    else
      80
    end
  end
end
