#
# Project
#
# Copyright (C) 2007  Windmill Media
# Copyright (c) 2006-2007 Lele Forzani <lele@windmill.it>
#
#
# = filename.rb
#
# <short descr>
#
# Author:: Lele Forzani <lele@windmill.it>
# Documentation::  Lele Forzani <lele@windmill.it>
# License::
#   This is NOT free software. License for using this software is granted
#   by written permission from Lele Forzani <lele@windmill.it>
# Revision:: $Id: $
#
#
# == Description
#
#
# == Annotations
#
#
# == Todo

=begin
 * Implements a minimal template engine with:
 * - variable substitution
 * - file inclusion
 *
 * variables in templates should be defined as:
 *   {var:variable-name}
 * 
 * includes as:
 *   {include:include-label}
 * 
 * variable substitution is performed by
 *   public function defineVar($var, $value)
 *
 * include label->file association is done by the method:
 *   public function defineInclude($name, $include)
 *
=end

class ICTemplate

private
# Array with the global variables
attr_accessor :variables
# Array with includes
attr_accessor :includes
# Array with the content of the template files
attr_accessor :templates

attr_accessor :templateRoot

public

def initialize(file, inline=false)
	@variables =  Hash.new
	@includes  =  Hash.new
	@templates = Hash.new

	if(!inline)
		@templateRoot = file
		templateLoad(@templateRoot)
	else
		@templateRoot = :inline
		templateLoad(file, :inline)
	
	end

end


def define(var, value)

	if(variables.has_key?(var.to_s))
		@variables[var.to_s] = value.to_s
		true
	else
		false	
	end
end

alias []= :define


def defineInclude(name, include)

	if(includes.has_key?(name))
		if(includes[name].nil?) 
			templateLoad(include)
			@includes[name] = include
		end
		true
	else
		false	
	end
end

def output
	# get template root
	text = templates[templateRoot]
		
	# perform variable substitution
	text = substituteIncludes(text)
	text = substituteVars(text)

	return text
end

# -------------------------------------------------------
protected

def templateLoad(file, inline=false)
	
	if(inline)
		@templates[:inline] = file
		templateParseVars(file)
		templateParseIncludes(file)
		return
	end
	
	unless(templates.has_key?(file))
		data =  IO.read(file)
		@templates[file] = data
		templateParseVars(data)
		templateParseIncludes(data)
	end
end

def templateParseVars(template)
	pvars = template.scan(/\{var:([\w\-]+)\}/)

	pvars.each { |varname| 
		if (!variables.has_key?(varname.first))
			variables[varname.first] = String.new
		end
	}

end

def templateParseIncludes(template)
	pincludes = template.scan(/\{include:([\w\-]+)\}/)	
	pincludes.each { |incname| 
		if (!includes.has_key?(incname.first))
			includes[incname.first] = String.new
		end
	}
end


def substituteVars(text)
	
	variables.each { |key, value|
		text = text.gsub(/\{var:#{key}\}/, value)
	}
	return text
end


def substituteIncludes(text)
	includes.each { |key, value|
		temp = templates[key]
		text = text.gsub(/\{include:#{key}\}/, value)
	}
	return text
end

end

# $URL: $
# $Rev: $
# $Author: $
# $Date: $
# $Id: $
